from xteam import asst, LOGS
from xteam.configs import Var

async def play_logs(event, title, duration, streamtype):
    if not Var.LOG_CHANNEL:
        return
    try:
        chat = await event.get_chat()
        chat_name = chat.title if hasattr(chat, 'title') else "Private Chat"
        user = await event.get_sender()
        user_mention = f"<a href='tg://user?id={user.id}'>{user.first_name}</a>"
        
        logger_text = (
            "<blockquote>"
            "🎵 Music Started\n\n"
            f"📌 Title: <code>{title}</code>\n"
            f"🕒 Duration: <code>{duration}</code>\n"
            f"🛰 Stream: <code>{streamtype}</code>"
            "</blockquote>\n"
            "<blockquote>"
            f"👥 Group: <code>{chat_name}</code>\n"
            f"🆔 ID: <code>{event.chat_id}</code>\n"
            f"👤 User: {user_mention}\n"
            f"🆔 User ID: <code>{user.id}</code>"
            "</blockquote>"
        )
        
        await asst.send_message(
            Var.LOG_CHANNEL,
            logger_text,
            parse_mode='html',
            link_preview=False
        )
    except Exception as e:
        LOGS.error(f"Gagal mengirim log: {e}")
        




from datetime import datetime as dt

from .. import udB


def get_stuff():
    # Fungsi ini sudah benar: mengambil data dari DB
    return udB.get_key("AFK_DB") or []


def add_afk(msg, media_type, media, start_time_str, timezone):
    """Menyimpan status AFK, termasuk media, waktu mulai, dan zona waktu."""
    # Fungsi ini sudah benar: menyimpan 5 item
    udB.set_key("AFK_DB", [msg, media_type, media, start_time_str, timezone])
    return


def is_afk():
    """Mengembalikan 5 nilai AFK yang tersimpan (msg, media_type, media, start_time_str, timezone)."""
    afk = get_stuff()
    
    # 🌟 PERBAIKAN: Lebih eksplisit dalam mengembalikan tuple
    if afk and len(afk) == 5:
        # Jika ada 5 item, kembalikan sebagai tuple.
        # Ini penting agar unpacking di fungsi utama (on_afk, remove_afk) berhasil
        return tuple(afk) 
    
    # Jika tidak ada atau jumlah item salah, kembalikan False
    return False


def del_afk():
    return udB.del_key("AFK_DB")
    



# Ultroid - UserBot
# Copyright (C) 2021-2024 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

from telethon import types
from telethon.extensions import markdown


class CustomMarkdown:
    @staticmethod
    def parse(text):
        text, entities = markdown.parse(text)
        for i, e in enumerate(entities):
            if isinstance(e, types.MessageEntityTextUrl):
                if e.url == "spoiler":
                    entities[i] = types.MessageEntitySpoiler(e.offset, e.length)
                elif e.url.startswith("emoji/"):
                    entities[i] = types.MessageEntityCustomEmoji(
                        e.offset, e.length, int(e.url.split("/")[1])
                    )
        return text, entities

    @staticmethod
    def unparse(text, entities):
        for i, e in enumerate(entities or []):
            if isinstance(e, types.MessageEntityCustomEmoji):
                entities[i] = types.MessageEntityTextUrl(
                    e.offset, e.length, f"emoji/{e.document_id}"
                )
            if isinstance(e, types.MessageEntitySpoiler):
                entities[i] = types.MessageEntityTextUrl(e.offset, e.length, "spoiler")
        return markdown.unparse(text, entities)



# Ultroid - UserBot
# Copyright (C) 2021-2023 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/xteam/blob/main/LICENSE>.

from importlib import util
from sys import modules

# for addons

configPaths = [
    "ub",
    "var",
    "support",
    "userbot",
    "telebot",
    "fridaybot",
    "uniborg.util",
    "telebot.utils",
    "userbot.utils",
    "userbot.events",
    "userbot.config",
    "fridaybot.utils",
    "fridaybot.Config",
    "userbot.uniborgConfig",
]


def load_addons(plugin_name):
    base_name = plugin_name.split("/")[-1].split("\\")[-1].replace(".py", "")
    if base_name.startswith("__"):
        return
    from xteam import fns

    from .. import HNDLR, LOGS, asst, udB, ultroid_bot
    from .._misc import _supporter as config
    from .._misc._assistant import asst_cmd, callback, in_pattern
    from .._misc._decorators import ultroid_cmd
    from .._misc._supporter import Config, admin_cmd, sudo_cmd
    from .._misc._wrappers import eod, eor
    from ..configs import Var
    from ..dB._core import HELP

    name = plugin_name.replace("/", ".").replace("\\", ".").replace(".py", "")
    spec = util.spec_from_file_location(name, plugin_name)
    mod = util.module_from_spec(spec)
    for path in configPaths:
        modules[path] = config
    modules["xteam.functions"] = fns
    mod.LOG_CHANNEL = udB.get_key("LOG_CHANNEL")
    mod.udB = udB
    mod.asst = asst
    mod.tgbot = asst
    mod.ultroid_bot = ultroid_bot
    mod.ub = ultroid_bot
    mod.bot = ultroid_bot
    mod.ultroid = ultroid_bot
    mod.borg = ultroid_bot
    mod.telebot = ultroid_bot
    mod.jarvis = ultroid_bot
    mod.friday = ultroid_bot
    mod.eod = eod
    mod.edit_delete = eod
    mod.LOGS = LOGS
    mod.in_pattern = in_pattern
    mod.hndlr = HNDLR
    mod.handler = HNDLR
    mod.HNDLR = HNDLR
    mod.CMD_HNDLR = HNDLR
    mod.Config = Config
    mod.Var = Var
    mod.eor = eor
    mod.edit_or_reply = eor
    mod.asst_cmd = asst_cmd
    mod.ultroid_cmd = ultroid_cmd
    mod.on_cmd = ultroid_cmd
    mod.callback = callback
    mod.Redis = udB.get_key
    mod.admin_cmd = admin_cmd
    mod.sudo_cmd = sudo_cmd
    mod.HELP = HELP.get("Addons", {})
    mod.CMD_HELP = HELP.get("Addons", {})

    spec.loader.exec_module(mod)
    modules[name] = mod
    doc = modules[name].__doc__.format(i=HNDLR) if modules[name].__doc__ else ""
    if "Addons" in HELP.keys():
        update_cmd = HELP["Addons"]
        try:
            update_cmd.update({base_name: doc})
        except BaseException:
            pass
    else:
        try:
            HELP.update({"Addons": {base_name: doc}})
        except BaseException as em:
            pass



# xteam - UserBot
# Copyright (C) 2024-2026 @xteam_cloner
#
# This file is a part of < https://codeberg.org/xteam-cloner/ >
# PLease read the GNU Affero General Public License in
# <>.

import os
import platform
import sys
from logging import INFO, WARNING, FileHandler, StreamHandler, basicConfig, getLogger
#from plugins.__init__ import OWNER_NAME
from .. import run_as_module
from ._extra import _ask_input

if run_as_module:
    from xteam.configs import Var
else:
    Var = None


def where_hosted():
    if os.getenv("DYNO"):
        return "heroku"
    if os.getenv("RAILWAY_STATIC_URL"):
        return "railway"
    if os.getenv("OKTETO_TOKEN"):
        return "okteto"
    if os.getenv("KUBERNETES_PORT"):
        return "qovery | kubernetes"
    if os.getenv("RUNNER_USER") or os.getenv("HOSTNAME"):
        if os.getenv("USER") == "codespace":
            return "codespace"
        return "github actions"
    if os.getenv("ANDROID_ROOT"):
        return "termux"
    if os.getenv("FLY_APP_NAME"):
        return "fly.io"
    return "VPS"


if run_as_module:
    from telethon import __version__
    from pytgcalls import __version__ as _ptgver_
    from telethon.tl.alltlobjects import LAYER

    from ..version import __version__ as __xteam__
    from ..version import ultroid_version

    file = f"userbot{sys.argv[6]}.log" if len(sys.argv) > 6 else "userbot.log"

    if os.path.exists(file):
        os.remove(file)

    HOSTED_ON = where_hosted()
    LOGS = getLogger("Xteam")
    TelethonLogger = getLogger("Telethon")
    TelethonLogger.setLevel(WARNING)

    _, v, __ = platform.python_version_tuple()

    if int(v) < 10:
        from ._extra import _fix_logging

        _fix_logging(FileHandler)

    _ask_input()

    _LOG_FORMAT = "%(asctime)s | %(name)s [%(levelname)s] : %(message)s"
    basicConfig(
        format=_LOG_FORMAT,
        level=INFO,
        datefmt="%m/%d/%Y, %H:%M:%S",
        handlers=[FileHandler(file), StreamHandler()],
    )
    try:

        import coloredlogs

        coloredlogs.install(level=None, logger=LOGS, fmt=_LOG_FORMAT)
    except ImportError:
        pass

    LOGS.info(
        """
                    -----------------------------------
                            Starting Deployment
                    -----------------------------------
    """
    )

    LOGS.info(f"Python version - {platform.python_version()}")
    LOGS.info(f"Xteam Version - {__xteam__}")
    LOGS.info(f"Telethon Version - {__version__} [Layer: {LAYER}]")
    LOGS.info(f"Userbot Version - {ultroid_version} [{HOSTED_ON}]")
    LOGS.info(f"PyTgCalls Version - {_ptgver_}")

    try:
        from safety.tools import *
    except ImportError:
        LOGS.error("'safety' package not found!")


import os
import re
import textwrap
import random
import aiofiles
import aiohttp
import asyncio
from PIL import Image, ImageDraw, ImageEnhance, ImageFilter, ImageFont, ImageOps, ImageChops
from youtubesearchpython.__future__ import VideosSearch

MUSIC_BOT_NAME = "Xteam Music"
YOUTUBE_IMG_URL = "https://telegra.ph/file/95d96663b73dbf278f28c.jpg"

# --- Pastikan Folder Cache & Thumbnail Ada ---
if not os.path.exists("cache"):
    os.makedirs("cache")
if not os.path.exists("thumbnail"):
    os.makedirs("thumbnail")

files = [] 
for filename in os.listdir("./thumbnail"): 
    if filename.endswith("png"): 
        files.append(filename[:-4])

if not files:
    files = ["default"]

def get_play_text(songname, artist, duration, from_user):
    if "|" in songname:
        judul_lagu, nama_artis = songname.split("|", 1)
    elif "-" in songname:
        judul_lagu, nama_artis = songname.split("-", 1)
    else:
        judul_lagu = songname
        nama_artis = artist

    return f"""
💡 Start Streaming!
🏷 **Artist :** {nama_artis.strip()}
🔍 **Title :** {judul_lagu.strip()}
🧭 **Duration :** {duration}
📢 **Channel :** {artist}
🎧 **Request :** {from_user}
"""

def get_play_queue(songname, artist, duration, from_user):
    if "|" in songname:
        judul_lagu, nama_artis = songname.split("|", 1)
    elif "-" in songname:
        judul_lagu, nama_artis = songname.split("-", 1)
    else:
        judul_lagu = songname
        nama_artis = artist

    return f"""
    🎧 Add to queue 🎥
    🏷 **Artist :** {nama_artis.strip()}
    🔍 **Title :** {judul_lagu.strip()}
    🧭 **Duration :** {duration}
    📢 **Channel :** {artist}
    🎧 **Request :** {from_user}"""
        

def changeImageSize(maxWidth, maxHeight, image):
    widthRatio = maxWidth / image.size[0]
    heightRatio = maxHeight / image.size[1]
    newWidth = int(widthRatio * image.size[0])
    newHeight = int(heightRatio * image.size[1])
    newImage = image.resize((newWidth, newHeight))
    return newImage

def add_corners(im):
    bigsize = (im.size[0] * 3, im.size[1] * 3)
    mask = Image.new('L', bigsize, 0)
    ImageDraw.Draw(mask).ellipse((0, 0) + bigsize, fill=255)
    mask = mask.resize(im.size, Image.LANCZOS)
    mask = ImageChops.darker(mask, im.split()[-1])
    im.putalpha(mask)

async def gen_thumb(videoid):
    anime = random.choice(files)
    if os.path.isfile(f"cache/{videoid}_{anime}.png"):
        return f"cache/{videoid}_{anime}.png"
    
    url = f"https://www.youtube.com/watch?v={videoid}"
    try:
        results = VideosSearch(url, limit=1)
        res = await results.next()
        if res["result"]:
            result = res["result"][0]
            title = re.sub("\W+", " ", result["title"]).title()
            duration = result.get("duration", "Unknown")
            thumbnail = result["thumbnails"][0]["url"].split("?")[0]
        else:
            return YOUTUBE_IMG_URL

        async with aiohttp.ClientSession() as session:
            async with session.get(thumbnail) as resp:
                if resp.status == 200:
                    content = await resp.read()
                    async with aiofiles.open(f"cache/thumb{videoid}.png", mode="wb") as f:
                        await f.write(content)

        if not os.path.exists(f"cache/thumb{videoid}.png"):
             return YOUTUBE_IMG_URL

        youtube = Image.open(f"cache/thumb{videoid}.png")
        bg_path = f"thumbnail/{anime}.png"
        bg = Image.open(bg_path) if os.path.exists(bg_path) else Image.new('RGBA', (1280, 720), (0, 0, 0, 255))

        image1 = changeImageSize(1280, 720, youtube)
        image2 = image1.convert("RGBA")
        background = image2.filter(filter=ImageFilter.BoxBlur(30))
        enhancer = ImageEnhance.Brightness(background)
        background = enhancer.enhance(0.6)
        
        cir_path = "thumbnail/IMG_20221129_201846_195.png"
        if os.path.exists(cir_path):
            cir = Image.open(cir_path)
            circle = changeImageSize(1280, 720, cir)
        else:
            circle = Image.new('RGBA', (1280, 720), (255, 255, 255, 0))

        image3 = changeImageSize(1280, 720, bg)
        image5 = image3.convert("RGBA")
        Image.alpha_composite(background, image5).save(f"cache/temp{videoid}.png")

        Xcenter = youtube.width / 2
        Ycenter = youtube.height / 2
        x1, y1, x2, y2 = Xcenter - 250, Ycenter - 250, Xcenter + 250, Ycenter + 250
        logo = youtube.crop((x1, y1, x2, y2))
        logo.thumbnail((520, 520), Image.LANCZOS)
        logo.save(f"cache/chop{videoid}.png")
        
        im = Image.open(f"cache/chop{videoid}.png").convert('RGBA')
        add_corners(im)
        im.save(f"cache/cropped{videoid}.png")

        crop_img = Image.open(f"cache/cropped{videoid}.png")
        logo = crop_img.convert("RGBA")
        logo.thumbnail((365, 365), Image.LANCZOS)
        width = int((1280 - 365)/ 2)
        background = Image.open(f"cache/temp{videoid}.png")
        background.paste(logo, (width + 2, 134), mask=logo)
        background.paste(circle, mask=circle)
        
        draw = ImageDraw.Draw(background)
        def get_font(path, size):
            return ImageFont.truetype(path, size) if os.path.exists(path) else ImageFont.load_default()

        font = get_font("thumbnail/font2.ttf", 45)
        arial = get_font("thumbnail/font2.ttf", 30)

        para = textwrap.wrap(title, width=32)
        try:
            if para[0]:
                l, t, r, b = draw.textbbox((0, 0), para[0], font=font)
                draw.text(((1280 - (r - l))/2, 530), para[0], fill="white", font=font)
            if len(para) > 1:
                l, t, r, b = draw.textbbox((0, 0), para[1], font=font)
                draw.text(((1280 - (r - l))/2, 580), para[1], fill="white", font=font)
        except: pass

        duration_text = f"Duration: {duration} Mins"
        l, t, r, b = draw.textbbox((0, 0), duration_text, font=arial)
        draw.text(((1280 - (r - l))/2, 660), duration_text, fill="white", font=arial)

        for temp in [f"cache/thumb{videoid}.png", f"cache/temp{videoid}.png", f"cache/chop{videoid}.png", f"cache/cropped{videoid}.png"]:
            if os.path.exists(temp): os.remove(temp)

        final_path = f"cache/{videoid}_{anime}.png"
        background.save(final_path)
        return final_path
    except Exception as e:
        print(f"Error gen_thumb: {e}")
        return YOUTUBE_IMG_URL
        



import asyncio
import math
import re
from telethon import events, Button
from xteam._misc._assistant import callback
from xteam import call_py

active_chats = {} 

MUSIC_BUTTONS = [
    [
        Button.inline("II", data="pauseit"),
        Button.inline("▷", data="resumeit")
    ],
    [
        Button.inline("‣‣I", data="skipit"),
        Button.inline("▢", data="stopit")
    ],
    [
        Button.inline("🗑", data="closeit")
    ]
]


def time_to_seconds(time_str):
    try:
        parts = str(time_str).split(':')
        if len(parts) == 2:
            return int(parts[0]) * 60 + int(parts[1])
        elif len(parts) == 3:
            return int(parts[0]) * 3600 + int(parts[1]) * 60 + int(parts[2])
    except:
        return 0
    return 0

def telegram_markup_timer(played, dur):
    played_sec = time_to_seconds(played)
    duration_sec = time_to_seconds(dur)
    
    if duration_sec == 0:
        return MUSIC_BUTTONS
        
    percentage = (played_sec / duration_sec) * 100
    umm = math.floor(percentage)
    
    if umm <= 10: bar = "◉—————————"
    elif 10 < umm < 20: bar = "—◉————————"
    elif 20 <= umm < 30: bar = "——◉———————"
    elif 30 <= umm < 40: bar = "———◉——————"
    elif 40 <= umm < 50: bar = "————◉—————"
    elif 50 <= umm < 60: bar = "—————◉————"
    elif 60 <= umm < 70: bar = "——————◉———"
    elif 70 <= umm < 80: bar = "———————◉——"
    elif 80 <= umm < 95: bar = "————————◉—"
    else: bar = "—————————◉"
    
    timer_row = [[Button.inline(f"{played} {bar} {dur}", data="timer_info")]]
    return timer_row + MUSIC_BUTTONS

async def timer_task(client, chat_id, message_id, duration):
    total_seconds = time_to_seconds(duration)
    played_seconds = 0
    active_chats[chat_id] = "playing"
    while chat_id in active_chats and played_seconds < total_seconds:
        await asyncio.sleep(15) 
        if active_chats.get(chat_id) == "paused":
            continue
        played_seconds += 15
        if played_seconds > total_seconds:
            played_seconds = total_seconds
        played_str = "{:02d}:{:02d}".format(*divmod(played_seconds, 60))
        markup = telegram_markup_timer(played_str, duration)
        try:
            await client.edit_message(chat_id, message_id, buttons=markup)
        except Exception:
            break
    active_chats.pop(chat_id, None)

@callback(data=re.compile(b"(pauseit|resumeit|stopit|skipit|closeit)"), owner=True)
async def music_manager(e):
    from plugins.music import skip_current_song 
    
    query = e.data.decode("utf-8")
    chat_id = e.chat_id
    try:
        if query == "pauseit":
            await call_py.pause(chat_id)
            active_chats[chat_id] = "paused"
            await e.answer("⏸ Paused")
        elif query == "resumeit":
            await call_py.resume(chat_id)
            active_chats[chat_id] = "playing"
            await e.answer("▶️ Resumed")
        elif query == "stopit":
            active_chats.pop(chat_id, None)
            await call_py.leave_call(chat_id)
            await e.delete()
        elif query == "skipit":
            await skip_current_song(e)
            await e.answer("⏭ Skipped")
        elif query == "closeit":
            await e.delete()
    except Exception as err:
        await e.answer(f"⚠️ Error: {str(err)}", alert=True)
        



# Man - UserBot
# Copyright (c) 2022 Man-Userbot
# Credits: @mrismanaziz || https://github.com/mrismanaziz
#
# This file is a part of < https://github.com/mrismanaziz/Man-Userbot/ >
# t.me/SharingUserbot & t.me/Lunatic0de


async def CHAT_TITLE(ctitle):
    string = ctitle
    font1 = list("𝔄𝔅ℭ𝔇𝔈𝔉𝔊ℌℑ𝔍𝔎𝔏𝔐𝔑𝔒𝔓𝔔ℜ𝔖𝔗𝔘𝔙𝔚𝔛𝔜ℨ")
    font2 = list("𝕬𝕭𝕮𝕯𝕰𝕱𝕲𝕳𝕴𝕵𝕶𝕷𝕸𝕹𝕺𝕻𝕼𝕽𝕾𝕿𝖀𝖁𝖂𝖃𝖄𝖅")
    font3 = list("𝓐𝓑𝓒𝓓𝓔𝓕𝓖𝓗𝓘𝓙𝓚𝓛𝓜𝓝𝓞𝓟𝓠𝓡𝓢𝓣𝓤𝓥𝓦𝓧𝓨𝓩")
    font4 = list("𝒜𝐵𝒞𝒟𝐸𝐹𝒢𝐻𝐼𝒥𝒦𝐿𝑀𝒩𝒪𝒫𝒬𝑅𝒮𝒯𝒰𝒱𝒲𝒳𝒴𝒵")
    font5 = list("𝔸𝔹ℂ𝔻𝔼𝔽𝔾ℍ𝕀𝕁𝕂𝕃𝕄ℕ𝕆ℙℚℝ𝕊𝕋𝕌𝕍𝕎𝕏𝕐ℤ")
    font6 = list("ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺ")
    font26 = list("𝐀𝐁𝐂𝐃𝐄𝐅𝐆𝐇𝐈𝐉𝐊𝐋𝐌𝐍𝐎𝐏𝐐𝐑𝐒𝐓𝐔𝐕𝐖𝐗𝐘𝐙")
    font27 = list("𝗔𝗕𝗖𝗗𝗘𝗙𝗚𝗛𝗜𝗝𝗞𝗟𝗠𝗡𝗢𝗣𝗤𝗥𝗦𝗧𝗨𝗩𝗪𝗫𝗬𝗭")
    font28 = list("𝘈𝘉𝘊𝘋𝘌𝘍𝘎𝘏𝘐𝘑𝘒𝘓𝘔𝘕𝘖𝘗𝘘𝘙𝘚𝘛𝘜𝘝𝘞𝘟𝘠𝘡")
    font29 = list("𝘼𝘽𝘾𝘿𝙀𝙁𝙂𝙃𝙄𝙅𝙆𝙇𝙈𝙉𝙊𝙋𝙌𝙍𝙎𝙏𝙐𝙑𝙒𝙓𝙔𝙕")
    font30 = list("𝙰𝙱𝙲𝙳𝙴𝙵𝙶𝙷𝙸𝙹𝙺𝙻𝙼𝙽𝙾𝙿𝚀𝚁𝚂𝚃𝚄𝚅𝚆𝚇𝚈𝚉")
    font1L = list("𝔞𝔟𝔠𝔡𝔢𝔣𝔤𝔥𝔦𝔧𝔨𝔩𝔪𝔫𝔬𝔭𝔮𝔯𝔰𝔱𝔲𝔳𝔴𝔵𝔶𝔷")
    font2L = list("𝖆𝖇𝖈𝖉𝖊𝖋𝖌𝖍𝖎𝖏𝖐𝖑𝖒𝖓𝖔𝖕𝖖𝖗𝖘𝖙𝖚𝖛𝖜𝖝𝖞𝖟")
    font3L = list("𝓪𝓫𝓬𝓭𝓮𝓯𝓰𝓱𝓲𝓳𝓴𝓵𝓶𝓷𝓸𝓹𝓺𝓻𝓼𝓽𝓾𝓿𝔀𝔁𝔂𝔃")
    font4L = list("𝒶𝒷𝒸𝒹𝑒𝒻𝑔𝒽𝒾𝒿𝓀𝓁𝓂𝓃𝑜𝓅𝓆𝓇𝓈𝓉𝓊𝓋𝓌𝓍𝓎𝓏")
    font5L = list("𝕒𝕓𝕔𝕕𝕖𝕗𝕘𝕙𝕚𝕛𝕜𝕝𝕞𝕟𝕠𝕡𝕢𝕣𝕤𝕥𝕦𝕧𝕨𝕩𝕪𝕫")
    font6L = list("ａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ")
    font27L = list("𝐚𝐛𝐜𝐝𝐞𝐟𝐠𝐡𝐢𝐣𝐤𝐥𝐦𝐧𝐨𝐩𝐪𝐫𝐬𝐭𝐮𝐯𝐰𝐱𝐲𝐳")
    font28L = list("𝗮𝗯𝗰𝗱𝗲𝗳𝗴𝗵𝗶𝗷𝗸𝗹𝗺𝗻𝗼𝗽𝗾𝗿𝘀𝘁𝘂𝘃𝘄𝘅𝘆𝘇")
    font29L = list("𝘢𝘣𝘤𝘥𝘦𝘧𝘨𝘩𝘪𝘫𝘬𝘭𝘮𝘯𝘰𝘱𝘲𝘳𝘴𝘵𝘶𝘷𝘸𝘹𝘺𝘻")
    font30L = list("𝙖𝙗𝙘𝙙𝙚𝙛𝙜𝙝𝙞𝙟𝙠𝙡𝙢𝙣𝙤𝙥𝙦𝙧𝙨𝙩𝙪𝙫𝙬𝙭𝙮𝙯")
    font31L = list("𝚊𝚋𝚌𝚍𝚎𝚏𝚐𝚑𝚒𝚓𝚔𝚕𝚖𝚗𝚘𝚙𝚚𝚛𝚜𝚝𝚞𝚟𝚠𝚡𝚢𝚣")
    normal = list("ABCDEFGHIJKLMNOPQRSTUVWXYZ")
    normalL = list("abcdefghijklmnopqrstuvwxyz")
    # small = list("ᴀʙᴄᴅᴇғɢʜɪᴊᴋʟᴍɴᴏᴘʀsᴛᴜᴠᴡxʏᴢ")
    cout = 0
    for XCB in font1:
        # string = string.replace(small[cout], normal[cout])
        string = string.replace(font1[cout], normal[cout])
        string = string.replace(font2[cout], normal[cout])
        string = string.replace(font3[cout], normal[cout])
        string = string.replace(font4[cout], normal[cout])
        string = string.replace(font5[cout], normal[cout])
        string = string.replace(font6[cout], normal[cout])
        string = string.replace(font26[cout], normal[cout])
        string = string.replace(font27[cout], normal[cout])
        string = string.replace(font28[cout], normal[cout])
        string = string.replace(font29[cout], normal[cout])
        string = string.replace(font30[cout], normal[cout])
        string = string.replace(font1L[cout], normalL[cout])
        string = string.replace(font2L[cout], normalL[cout])
        string = string.replace(font3L[cout], normalL[cout])
        string = string.replace(font4L[cout], normalL[cout])
        string = string.replace(font5L[cout], normalL[cout])
        string = string.replace(font6L[cout], normalL[cout])
        string = string.replace(font27L[cout], normalL[cout])
        string = string.replace(font28L[cout], normalL[cout])
        string = string.replace(font29L[cout], normalL[cout])
        string = string.replace(font30L[cout], normalL[cout])
        string = string.replace(font31L[cout], normalL[cout])
        cout += 1
    return string



# xteam-urbot 
# Copyright (C) 2021-2025 Xteam_cloner
#
# PLease read the GNU Affero General Public License in
# <https://github.com/xteam-cloner/xteam/blob/main/LICENSE>.

from . import *
import os
import sys
import time
import asyncio 
from pytgcalls import PyTgCalls
from telethon import TelegramClient 
from telethon.errors.rpcerrorlist import AuthKeyDuplicatedError
from telethon.sessions import StringSession
from telethon.network.connection.tcpabridged import ConnectionTcpAbridged
import xteam 
from .startup.connections import validate_session
from strings import get_string
from .fns.helper import bash, time_formatter, updater
from .startup.funcs import (
    WasItRestart,
    autopilot,
    customize,
    fetch_ann, 
    plug,
    ready,
    startup_stuff,
)
from .startup.loader import load_other_plugins 

def register_vc_handlers():
    """
    Mendaftarkan unified_update_handler ke instance PyTgCalls.
    Fungsi ini dipindahkan ke sini (__main__.py) untuk memecahkan circular dependency.
    """
    # Impor handler-nya langsung dari modul handlers.py
    from xteam.handlers import unified_update_handler 
    from xteam import call_py
    
# Pastikan call_py sudah ada dan diinisialisasi
    if call_py:
        call_py.on_update()(unified_update_handler)
    else:
        LOGS.warning("call_py not available, handler registration skipped.")


async def main_async():
    
    try:
        from apscheduler.schedulers.asyncio import AsyncIOScheduler
    except ImportError:
        AsyncIOScheduler = None

    if (
        udB.get_key("UPDATE_ON_RESTART")
        and os.path.exists(".git")
        and await updater()
    ):
        await bash("bash installer.sh") 
        os.execl(sys.executable, sys.executable, "-m", "xteam")

    await startup_stuff()

    ultroid_bot.me.phone = None 

    if not ultroid_bot.me.bot:
        udB.set_key("OWNER_ID", ultroid_bot.uid)

    LOGS.info("Initialising...")

    await autopilot()

    pmbot = udB.get_key("PMBOT")
    manager = udB.get_key("MANAGER")
    addons = udB.get_key("ADDONS") or Var.ADDONS
    vcbot_enabled = udB.get_key("VCBOT") or Var.VCBOT
    
    if HOSTED_ON == "okteto":
        vcbot_enabled = False

    if (HOSTED_ON == "termux" or udB.get_key("LITE_DEPLOY")) and udB.get_key(
        "EXCLUDE_OFFICIAL"
    ) is None:
        _plugins = "autocorrect autopic audiotools compressor forcesubscribe fedutils gdrive glitch instagram nsfwfilter nightmode pdftools profanityfilter writer youtube"
        udB.set_key("EXCLUDE_OFFICIAL", _plugins)

    call_py = None
    
    if vcbot_enabled:
        VC_SESSION = udB.get_key("VC_SESSION") or Var.VC_SESSION
        
        if VC_SESSION:
            session = validate_session(VC_SESSION)
        elif HOSTED_ON == "heroku":
            LOGS.warning("VCBOT enabled but VC_SESSION is missing. VC Bot disabled.")
            vcbot_enabled = False
            call_py = None
            session = None
        else:
            session = "xteam-musicubot"
            LOGS.info("VCBOT enabled but VC_SESSION missing. Trying local session.")

        if session:
            try:
                bot = TelegramClient(
                    session=session,
                    api_id=Var.API_ID,
                    api_hash=Var.API_HASH,
                    connection=ConnectionTcpAbridged,
                    auto_reconnect=True,
                    connection_retries=None,
                )
                
                await bot.start() 
                vc_me = await bot.get_me()
                full_name = vc_me.first_name
                if vc_me.last_name:
                    full_name += f" {vc_me.last_name}"
                LOGS.info(f"Assistans Start as {full_name}") 
                
                call_py = PyTgCalls(bot)
                await call_py.start()
                LOGS.info("PyTgCalls Client started successfully.")
                
                xteam.bot = bot
                xteam.call_py = call_py
        
            except (AuthKeyDuplicatedError, EOFError):
                LOGS.info(get_string("py_c3"))
                udB.del_key("VC_SESSION")
                call_py = None
            except Exception as er:
                LOGS.info("While creating or starting PyTgCalls Client for VC. VC Bot disabled.")
                LOGS.exception(er)
                call_py = None

    await load_other_plugins(
        addons=addons, 
        pmbot=pmbot, 
        manager=manager, 
        vcbot=call_py
    )

    suc_msg = """
            ----------------------------------------------------------------------
                xteam-urbot has been deployed! Visit @xteam_cloner for updates!!
            ----------------------------------------------------------------------
    """

    plugin_channels = udB.get_key("PLUGIN_CHANNEL")

    await customize()

    if plugin_channels:
        await plug(plugin_channels)

    if not udB.get_key("LOG_OFF"):
        await ready()

    await WasItRestart(udB)

    try:
        cleanup_cache()
    except BaseException:
        pass

    LOGS.info(
        f"Took {time_formatter((time.time() - start_time)*1000)} to start "
    )
    LOGS.info(suc_msg)


if __name__ == "__main__":
    
    ultroid_bot.start() 

    ultroid_bot.loop.create_task(main_async()) 
    
    asst.run()
    



from .chattitle import CHAT_TITLE
from .controls import skip_current_song, skip_item
from .queues import QUEUE, add_to_queue, clear_queue, get_queue, pop_an_item
from .thumbnail import gen_thumb, get_play_text, get_play_queue
from .ytdl import ytdl, ytsearch, cleanup_file, get_playlist_ids
from .join_call import join_call
from .markups import telegram_markup_timer, MUSIC_BUTTONS
from .joiner import AssistantAdd
from .play_logs import play_logs



# Man - UserBot
# Copyright (c) 2022 Man-Userbot
# Credits: @mrismanaziz || https://github.com/mrismanaziz
# This file is a part of < https://github.com/mrismanaziz/Man-Userbot/ >
# t.me/SharingUserbot & t.me/Lunatic0de

QUEUE = {}

def add_to_queue(chat_id, songname, url, duration, thumb_url, videoid, artist, from_user, is_video):
    if chat_id not in QUEUE:
        QUEUE[chat_id] = []
    QUEUE[chat_id].append([songname, url, duration, thumb_url, videoid, artist, from_user, is_video])
    return len(QUEUE[chat_id])

def get_queue(chat_id):
    if chat_id in QUEUE:
        return QUEUE[chat_id]
    return 0

def pop_an_item(chat_id):
    if chat_id not in QUEUE:
        return 0
    chat_queue = QUEUE[chat_id]
    if chat_queue:
        chat_queue.pop(0)
        return 1
    return 0

def clear_queue(chat_id: int):
    if chat_id in QUEUE:
        QUEUE.pop(chat_id)
        return 1
    return 0
    



# Ultroid - UserBot
# Copyright (C) 2021-2023 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

from .. import udB


def get_muted():
    return udB.get_key("MUTE") or {}


def mute(chat, id):
    ok = get_muted()
    if ok.get(chat):
        if id not in ok[chat]:
            ok[chat].append(id)
    else:
        ok.update({chat: [id]})
    return udB.set_key("MUTE", ok)


def unmute(chat, id):
    ok = get_muted()
    if ok.get(chat) and id in ok[chat]:
        ok[chat].remove(id)
    return udB.set_key("MUTE", ok)


def is_muted(chat, id):
    ok = get_muted()
    return bool(ok.get(chat) and id in ok[chat])



# Ultroid - UserBot
# Copyright (C) 2021-2025 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

import os
import subprocess
import sys
from shutil import rmtree

from decouple import config
from git import Repo

from .. import *
from ..dB._core import HELP
from ..loader import Loader
from . import *
from .utils import load_addons


def _after_load(loader, module, plugin_name=""):
    if not module or plugin_name.startswith("_"):
        return
    from strings import get_help

    if doc_ := get_help(plugin_name) or module.__doc__:
        try:
            doc = doc_.format(i=HNDLR)
        except Exception as er:
            loader._logger.exception(er)
            loader._logger.info(f"Error in {plugin_name}: {module}")
            return
        if loader.key in HELP.keys():
            update_cmd = HELP[loader.key]
            try:
                update_cmd.update({plugin_name: doc})
            except BaseException as er:
                loader._logger.exception(er)
        else:
            try:
                HELP.update({loader.key: {plugin_name: doc}})
            except BaseException as em:
                loader._logger.exception(em)


# *** PERUBAHAN KRITIS 1: Tambahkan vcUserClient sebagai argumen ***
async def load_other_plugins(addons=None, pmbot=None, manager=None, vcbot=None):

    # for official
    _exclude = udB.get_key("EXCLUDE_OFFICIAL") or config("EXCLUDE_OFFICIAL", None)
    _exclude = _exclude.split() if _exclude else []

    # "INCLUDE_ONLY" was added to reduce Big List in "EXCLUDE_OFFICIAL" Plugin
    _in_only = udB.get_key("INCLUDE_ONLY") or config("INCLUDE_ONLY", None)
    _in_only = _in_only.split() if _in_only else []
    Loader().load(include=_in_only, exclude=_exclude, after_load=_after_load)

    # for assistant
    if not USER_MODE and not udB.get_key("DISABLE_AST_PLUGINS"):
        _ast_exc = ["pmbot"]
        if _in_only and "games" not in _in_only:
            _ast_exc.append("games")
        Loader(path="assistant").load(
            log=False, exclude=_ast_exc, after_load=_after_load
        )

    # for addons
    if addons:
        if url := udB.get_key("ADDONS_URL"):
            subprocess.run(f"git clone -q {url} addons", shell=True)
        if os.path.exists("addons") and not os.path.exists("addons/.git"):
            rmtree("addons")
        if not os.path.exists("addons"):
            subprocess.run(
                f"git clone -q -b {Repo().active_branch} https://github.com/TeamUltroid/UltroidAddons.git addons",
                shell=True,
            )
        else:
            subprocess.run("cd addons && git pull -q && cd ..", shell=True)

        if not os.path.exists("addons"):
            subprocess.run(
                "git clone -q https://github.com/TeamUltroid/UltroidAddons.git addons",
                shell=True,
            )
        if os.path.exists("addons/addons.txt"):
            # generally addons req already there so it won't take much time
            # subprocess.run(
            #        "rm -rf /usr/local/lib/python3.*/site-packages/pip/_vendor/.wh*"
            #    )
            subprocess.run(
                f"{sys.executable} -m pip install --no-cache-dir -q -r ./addons/addons.txt",
                shell=True,
            )

        _exclude = udB.get_key("EXCLUDE_ADDONS")
        _exclude = _exclude.split() if _exclude else []
        _in_only = udB.get_key("INCLUDE_ADDONS")
        _in_only = _in_only.split() if _in_only else []

        Loader(path="addons", key="Addons").load(
            func=load_addons,
            include=_in_only,
            exclude=_exclude,
            after_load=_after_load,
            load_all=True,
        )

    if not USER_MODE:
        # group manager
        if manager:
            Loader(path="assistant/manager", key="Group Manager").load()

        # chat via assistant
        if pmbot:
            Loader(path="assistant/pmbot.py").load(log=False)

    


import logging
from xteam import call_py, bot as client

logger = logging.getLogger(__name__)


@call_py.on_update()
async def log_updates(client, update):
    logger.debug(f"Update diterima: {type(update).__name__}")
    



# Ultroid - UserBot
# Copyright (C) 2021-2023 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/xteam/blob/main/LICENSE>.

import asyncio
import os
import random
import shutil
import time
import platform
from random import randint
from telethon import __version__ as tver
from pyrogram import __version__ as pver
from platform import python_version
from pytgcalls import __version__ as pyt
from xteam.version import ultroid_version as UltVer
from xteam.version import __version__ as xtver
from xteam.configs import Var

from . import HOSTED_ON
try:
    from pytz import timezone
except ImportError:
    timezone = None

from telethon.errors import (
    ChannelsTooMuchError,
    ChatAdminRequiredError,
    MessageIdInvalidError,
    MessageNotModifiedError,
    UserNotParticipantError,
)
from telethon.tl.custom import Button
from telethon.tl.functions.channels import (
    CreateChannelRequest,
    EditAdminRequest,
    EditPhotoRequest,
    InviteToChannelRequest,
)
from telethon.tl.functions.contacts import UnblockRequest
from telethon.tl.types import (
    ChatAdminRights,
    ChatPhotoEmpty,
    InputChatUploadedPhoto,
    InputMessagesFilterDocument,
)
from telethon.utils import get_peer_id
from decouple import config, RepositoryEnv
from .. import LOGS, ULTConfig
from ..fns.helper import download_file, inline_mention, updater
#from .._misc._assistant import callback
db_url = 0


async def autoupdate_local_database():
    from .. import asst, udB, ultroid_bot

    global db_url
    db_url = (
        udB.get_key("TGDB_URL") or Var.TGDB_URL or ultroid_bot._cache.get("TGDB_URL")
    )
    if db_url:
        _split = db_url.split("/")
        _channel = _split[-2]
        _id = _split[-1]
        try:
            await asst.edit_message(
                int(_channel) if _channel.isdigit() else _channel,
                message=_id,
                file="database.json",
                text="**Do not delete this file.**",
            )
        except MessageNotModifiedError:
            return
        except MessageIdInvalidError:
            pass
    try:
        LOG_CHANNEL = (
            udB.get_key("LOG_CHANNEL")
            or Var.LOG_CHANNEL
            or asst._cache.get("LOG_CHANNEL")
            or "me"
        )
        msg = await asst.send_message(
            LOG_CHANNEL, "**Do not delete this file.**", file="database.json"
        )
        asst._cache["TGDB_URL"] = msg.message_link
        udB.set_key("TGDB_URL", msg.message_link)
    except Exception as ex:
        LOGS.error(f"Error on autoupdate_local_database: {ex}")


def update_envs():
    """Update Var. attributes to udB"""
    from .. import udB
    _envs = [*list(os.environ)]
    if ".env" in os.listdir("."):
        [_envs.append(_) for _ in list(RepositoryEnv(config._find_file(".")).data)]
    for envs in _envs:
        if (
            envs in ["LOG_CHANNEL", "BOT_TOKEN", "BOTMODE", "DUAL_MODE", "language"]
            or envs in udB.keys()
        ):
            if _value := os.environ.get(envs):
                udB.set_key(envs, _value)
            else:
                udB.set_key(envs, config.config.get(envs))


async def startup_stuff():
    from .. import udB

    x = ["resources/auth", "resources/downloads"]
    for x in x:
        if not os.path.isdir(x):
            os.mkdir(x)

    CT = udB.get_key("CUSTOM_THUMBNAIL")
    if CT:
        path = "resources/extras/thumbnail.jpg"
        ULTConfig.thumb = path
        try:
            await download_file(CT, path)
        except Exception as er:
            LOGS.exception(er)
    elif CT is False:
        ULTConfig.thumb = None
    GT = udB.get_key("GDRIVE_AUTH_TOKEN")
    if GT:
        with open("resources/auth/gdrive_creds.json", "w") as t_file:
            t_file.write(GT)

    if udB.get_key("AUTH_TOKEN"):
        udB.del_key("AUTH_TOKEN")

    MM = udB.get_key("MEGA_MAIL")
    MP = udB.get_key("MEGA_PASS")
    if MM and MP:
        with open(".megarc", "w") as mega:
            mega.write(f"[Login]\nUsername = {MM}\nPassword = {MP}")

    TZ = udB.get_key("TIMEZONE")
    if TZ and timezone:
        try:
            timezone(TZ)
            os.environ["TZ"] = TZ
            time.tzset()
        except AttributeError as er:
            LOGS.debug(er)
        except BaseException:
            LOGS.critical(
                "Incorrect Timezone ,\nCheck Available Timezone From Here https://graph.org/Ultroid-06-18-2\nSo Time is Default UTC"
            )
            os.environ["TZ"] = "UTC"
            time.tzset()


async def autobot():
    from .. import udB, ultroid_bot

    if udB.get_key("BOT_TOKEN"):
        return
    await ultroid_bot.start()
    LOGS.info("MAKING A TELEGRAM BOT FOR YOU AT @BotFather, Kindly Wait")
    who = ultroid_bot.me
    name = who.first_name + "Bot"
    if who.username:
        username = who.username + "_bot"
    else:
        username = "xteam_" + (str(who.id))[5:] + "_bot"
    bf = "@BotFather"
    await ultroid_bot(UnblockRequest(bf))
    await ultroid_bot.send_message(bf, "/cancel")
    await asyncio.sleep(1)
    await ultroid_bot.send_message(bf, "/newbot")
    await asyncio.sleep(1)
    isdone = (await ultroid_bot.get_messages(bf, limit=1))[0].text
    if isdone.startswith("That I cannot do.") or "20 bots" in isdone:
        LOGS.critical(
            "Please make a Bot from @BotFather and add it's token in BOT_TOKEN, as an env var and restart me."
        )
        import sys

        sys.exit(1)
    await ultroid_bot.send_message(bf, name)
    await asyncio.sleep(1)
    isdone = (await ultroid_bot.get_messages(bf, limit=1))[0].text
    if not isdone.startswith("Good."):
        await ultroid_bot.send_message(bf, "My Assistant Bot")
        await asyncio.sleep(1)
        isdone = (await ultroid_bot.get_messages(bf, limit=1))[0].text
        if not isdone.startswith("Good."):
            LOGS.critical(
                "Please make a Bot from @BotFather and add it's token in BOT_TOKEN, as an env var and restart me."
            )
            import sys

            sys.exit(1)
    await ultroid_bot.send_message(bf, username)
    await asyncio.sleep(1)
    isdone = (await ultroid_bot.get_messages(bf, limit=1))[0].text
    await ultroid_bot.send_read_acknowledge("botfather")
    if isdone.startswith("Sorry,"):
        ran = randint(1, 100)
        username = "xteam_" + (str(who.id))[6:] + str(ran) + "_bot"
        await ultroid_bot.send_message(bf, username)
        await asyncio.sleep(1)
        isdone = (await ultroid_bot.get_messages(bf, limit=1))[0].text
    if isdone.startswith("Done!"):
        token = isdone.split("`")[1]
        udB.set_key("BOT_TOKEN", token)
        await enable_inline(ultroid_bot, username)
        LOGS.info(
            f"Done. Successfully created @{username} to be used as your assistant bot!"
        )
    else:
        LOGS.info(
            "Please Delete Some Of your Telegram bots at @Botfather or Set Var BOT_TOKEN with token of a bot"
        )

        import sys

        sys.exit(1)


async def autopilot():
    from .. import asst, udB, ultroid_bot

    channel = udB.get_key("LOG_CHANNEL")
    new_channel = None
    if channel:
        try:
            chat = await ultroid_bot.get_entity(channel)
        except BaseException as err:
            LOGS.exception(err)
            udB.del_key("LOG_CHANNEL")
            channel = None
    if not channel:

        async def _save(exc):
            udB._cache["LOG_CHANNEL"] = ultroid_bot.me.id
            await asst.send_message(
                ultroid_bot.me.id, f"Failed to Create Log Channel due to {exc}.."
            )

        if ultroid_bot._bot:
            msg_ = "'LOG_CHANNEL' not found! Add it in order to use 'BOTMODE'"
            LOGS.error(msg_)
            return await _save(msg_)
        LOGS.info("Creating a Log Channel for You!")
        try:
            r = await ultroid_bot(
                CreateChannelRequest(
                    title="My Userbot Logs",
                    about="My Userbot Log Group\n\n Join @xteam_cloner",
                    megagroup=True,
                ),
            )
        except ChannelsTooMuchError as er:
            LOGS.critical(
                "You Are in Too Many Channels & Groups , Leave some And Restart The Bot"
            )
            return await _save(str(er))
        except BaseException as er:
            LOGS.exception(er)
            LOGS.info(
                "Something Went Wrong , Create A Group and set its id on config var LOG_CHANNEL."
            )

            return await _save(str(er))
        new_channel = True
        chat = r.chats[0]
        channel = get_peer_id(chat)
        udB.set_key("LOG_CHANNEL", channel)
    assistant = True
    try:
        await ultroid_bot.get_permissions(int(channel), asst.me.username)
    except UserNotParticipantError:
        try:
            await ultroid_bot(InviteToChannelRequest(int(channel), [asst.me.username]))
        except BaseException as er:
            LOGS.info("Error while Adding Assistant to Log Channel")
            LOGS.exception(er)
            assistant = False
    except BaseException as er:
        assistant = False
        LOGS.exception(er)
    if assistant and new_channel:
        try:
            achat = await asst.get_entity(int(channel))
        except BaseException as er:
            achat = None
            LOGS.info("Error while getting Log channel from Assistant")
            LOGS.exception(er)
        if achat and not achat.admin_rights:
            rights = ChatAdminRights(
                add_admins=True,
                invite_users=True,
                change_info=True,
                ban_users=True,
                delete_messages=True,
                pin_messages=True,
                anonymous=False,
                manage_call=True,
            )
            try:
                await ultroid_bot(
                    EditAdminRequest(
                        int(channel), asst.me.username, rights, "Assistant"
                    )
                )
            except ChatAdminRequiredError:
                LOGS.info(
                    "Failed to promote 'Assistant Bot' in 'Log Channel' due to 'Admin Privileges'"
                )
            except BaseException as er:
                LOGS.info("Error while promoting assistant in Log Channel..")
                LOGS.exception(er)
    if isinstance(chat.photo, ChatPhotoEmpty):
        photo, _ = await download_file(
            "https://files.catbox.moe/k9ljse.jpg", "channelphoto.jpg"
        )
        ll = await ultroid_bot.upload_file(photo)
        try:
            await ultroid_bot(
                EditPhotoRequest(int(channel), InputChatUploadedPhoto(ll))
            )
        except BaseException as er:
            LOGS.exception(er)
        os.remove(photo)


# customize assistant


async def customize():
    from .. import asst, udB, ultroid_bot

    rem = None
    try:
        chat_id = udB.get_key("LOG_CHANNEL")
        if asst.me.photo:
            return
        LOGS.info("Customising Ur Assistant Bot in @BOTFATHER")
        UL = f"@{asst.me.username}"
        if not ultroid_bot.me.username:
            sir = ultroid_bot.me.first_name
        else:
            sir = f"@{ultroid_bot.me.username}"
        file = random.choice(
            [
                "resources/extras/8189450f-de7f-4582-ba94-f8ec2d928b31.jpeg",
            ]
        )
        if not os.path.exists(file):
            file, _ = await download_file(file, "profile.jpg")
            rem = True
        msg = await asst.send_message(
            chat_id, "**Auto Customisation** Started on @Botfather"
        )
        await asyncio.sleep(1)
        await ultroid_bot.send_message("botfather", "/cancel")
        await asyncio.sleep(1)
        await ultroid_bot.send_message("botfather", "/setuserpic")
        await asyncio.sleep(1)
        isdone = (await ultroid_bot.get_messages("botfather", limit=1))[0].text
        if isdone.startswith("Invalid bot"):
            LOGS.info("Error while trying to customise assistant, skipping...")
            return
        await ultroid_bot.send_message("botfather", UL)
        await asyncio.sleep(1)
        await ultroid_bot.send_file("botfather", file)
        await asyncio.sleep(2)
        await ultroid_bot.send_message("botfather", "/setabouttext")
        await asyncio.sleep(1)
        await ultroid_bot.send_message("botfather", UL)
        await asyncio.sleep(1)
        await ultroid_bot.send_message(
            "botfather", f"🥀 Hello !! I'm Assistant Bot of {sir}"
        )
        await asyncio.sleep(2)
        await ultroid_bot.send_message("botfather", "/setdescription")
        await asyncio.sleep(1)
        await ultroid_bot.send_message("botfather", UL)
        await asyncio.sleep(1)
        await ultroid_bot.send_message(
            "botfather",
            f"🥀 Powerful Urbot Assistant Bot 🥀\n🥀 Master ~ {sir} 🥀\n\n🥀 Powered By ~ @xteam_cloner 🥀",
        )
        await asyncio.sleep(2)
        await msg.edit("Completed **Auto Customisation** at @BotFather.")
        if rem:
            os.remove(file)
        LOGS.info("Customisation Done")
    except Exception as e:
        LOGS.exception(e)


async def plug(plugin_channels):
    from .. import ultroid_bot
    from .utils import load_addons, closeit

    if ultroid_bot._bot:
        LOGS.info("Plugin Channels can't be used in 'BOTMODE'")
        return
    if os.path.exists("addons") and not os.path.exists("addons/.git"):
        shutil.rmtree("addons")
    if not os.path.exists("addons"):
        os.mkdir("addons")
    if not os.path.exists("addons/__init__.py"):
        with open("addons/__init__.py", "w") as f:
            f.write("from modules import *\n\nbot = ultroid_bot")
    LOGS.info("• Loading Plugins from Plugin Channel(s) •")
    for chat in plugin_channels:
        LOGS.info(f"{'•'*4} {chat}")
        try:
            async for x in ultroid_bot.iter_messages(
                chat, search=".py", filter=InputMessagesFilterDocument, wait_time=10
            ):
                plugin = "addons/" + x.file.name.replace("_", "-").replace("|", "-")
                if not os.path.exists(plugin):
                    await asyncio.sleep(0.6)
                    if x.text == "#IGNORE":
                        continue
                    plugin = await x.download_media(plugin)
                    try:
                        load_addons(plugin)
                    except Exception as e:
                        LOGS.info(f"Userbot - PLUGIN_CHANNEL - ERROR - {plugin}")
                        LOGS.exception(e)
                        os.remove(plugin)
        except Exception as er:
            LOGS.exception(er)


# some stuffs


async def fetch_ann():
    from .. import asst, udB
    from ..fns.tools import async_searcher
    
    
    get_ = udB.get_key("OLDANN") or []
    chat_id = udB.get_key("LOG_CHANNEL")
    try:
        updts = await async_searcher(
            "https://ultroid-api.vercel.app/announcements", post=True, re_json=True
        )
        for upt in updts:
            key = list(upt.keys())[0]
            if key not in get_:
                cont = upt[key]
                if isinstance(cont, dict) and cont.get("lang"):
                    if cont["lang"] != (udB.get_key("language") or "en"):
                        continue
                    cont = cont["msg"]
                if isinstance(cont, str):
                    await asst.send_message(chat_id, cont)
                elif isinstance(cont, dict) and cont.get("chat"):
                    await asst.forward_messages(chat_id, cont["msg_id"], cont["chat"])
                else:
                    LOGS.info(cont)
                    LOGS.info(
                        "Invalid Type of Announcement Detected!\nMake sure you are on latest version.."
                    )
                get_.append(key)
        udB.set_key("OLDANN", get_)
    except Exception as er:
        LOGS.exception(er)


async def ready():
    from .. import asst, udB, ultroid_bot
    
    
    chat_id = udB.get_key("LOG_CHANNEL")
    spam_sent = None
    
    if not udB.get_key("INIT_DEPLOY"):
        MSG = """ **Thanks for Deploying Userbot!**
• Here, are the Some Basic stuff from, where you can Know, about its Usage."""
        PHOTO = "https://files.catbox.moe/k9ljse.jpg"
        BTTS = [[Button.inline("• Click to Start •", "initft_2")]]
        udB.set_key("INIT_DEPLOY", "Done")

    else:
        MSG = f"<blockquote>▰▰▰▰▰▰▰▰▰▰▰▰▰▰▰\n🔥 xᴛᴇᴀᴍ-ᴜʀʙᴏᴛ ᴀᴄᴛɪᴠᴀᴛᴇᴅ 🔥\n Owner : {ultroid_bot.full_name}\n xteam-urbot : {UltVer} {HOSTED_ON}\n XteamLib : {xtver}\n Telethon : {tver}\n Python : {platform.python_version()}\n PyTgCalls : {pyt}\n Support : @xteam_cloner\n▰▰▰▰▰▰▰▰▰▰▰▰▰▰▰</blockquote>"        
        
        PHOTO = "https://files.catbox.moe/k9ljse.jpg"
        
        BTTS = [
            [
                Button.url("Support Channel", "https://t.me/xteam_cloner"),
                Button.inline("Close", "closeit")
            ]
        ]
        
        prev_spam = udB.get_key("LAST_UPDATE_LOG_SPAM")
        if prev_spam:
            try:
                await ultroid_bot.delete_messages(chat_id, int(prev_spam))
            except Exception as E:
                LOGS.info("Error while Deleting Previous Update Message :" + str(E))
        
        if await updater():
            BTTS[0].append(Button.inline("Update Available", "updtavail"))

    try:
        spam_sent = await asst.send_message(chat_id, MSG, file=PHOTO, buttons=BTTS, parse_mode="html")
    except ValueError as e:
        try:
            await (await ultroid_bot.send_message(chat_id, str(e))).delete()
            spam_sent = await asst.send_message(chat_id, MSG, file=PHOTO, buttons=BTTS, parse_mode="html")
        except Exception as g:
            LOGS.info(g)
    except Exception as el:
        LOGS.info(el)
        try:
            spam_sent = await ultroid_bot.send_message(chat_id, MSG, file=PHOTO, parse_mode="html")
        except Exception as ef:
            LOGS.exception(ef)
            
    if spam_sent and not spam_sent.media:
        udB.set_key("LAST_UPDATE_LOG_SPAM", spam_sent.id)


async def WasItRestart(udb):
    key = udb.get_key("_RESTART")
    if not key:
        return
    from .. import asst, ultroid_bot

    try:
        data = key.split("_")
        who = asst if data[0] == "bot" else ultroid_bot
        await who.edit_message(
            int(data[1]), int(data[2]), "__```Restarted Successfully.```__"
        )
    except Exception as er:
        LOGS.exception(er)
    udb.del_key("_RESTART")


def _version_changes(udb):
    for _ in [
        "BOT_USERS",
        "BOT_BLS",
        "VC_SUDOS",
        "SUDOS",
        "CLEANCHAT",
        "LOGUSERS",
        "PLUGIN_CHANNEL",
        "CH_SOURCE",
        "CH_DESTINATION",
        "BROADCAST",
    ]:
        key = udb.get_key(_)
        if key and str(key)[0] != "[":
            key = udb.get(_)
            new_ = [
                int(z) if z.isdigit() or (z.startswith("-") and z[1:].isdigit()) else z
                for z in key.split()
            ]
            udb.set_key(_, new_)


async def enable_inline(ultroid_bot, username):
    bf = "BotFather"
    await ultroid_bot.send_message(bf, "/setinline")
    await asyncio.sleep(1)
    await ultroid_bot.send_message(bf, f"@{username}")
    await asyncio.sleep(1)
    await ultroid_bot.send_message(bf, "Search")
    await ultroid_bot.send_read_acknowledge(bf)



import os
import asyncio
import yt_dlp
import logging
import glob
from typing import Tuple, Union, Any, List
from youtubesearchpython import VideosSearch

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

BASE_PATH = os.path.dirname(os.path.abspath(__file__))
DOWNLOAD_DIR = os.path.join(BASE_PATH, "downloads")
COOKIES_FILE_PATH = os.path.join(BASE_PATH, "cookies.txt")

if not os.path.exists(DOWNLOAD_DIR):
    os.makedirs(DOWNLOAD_DIR)

def ytsearch(query: str) -> Union[List[Any], int]:
    try:
        search = VideosSearch(query, limit=1).result()
        if not search["result"]:
            return 0
        data = search["result"][0]
        return [
            data["title"],
            data["link"],
            data["duration"],
            data["thumbnails"][0]["url"],
            data["id"],
            data["channel"]["name"]
        ]
    except Exception as e:
        logger.error(f"Search Error: {e}")
        return 0

async def ytdl(url: str, video_mode: bool = False, mode: str = None) -> Tuple[int, str]:
    loop = asyncio.get_running_loop()

    def download_sync():
        selected_mode = mode or ("video" if video_mode else "audio")
        
        opts = {
            "quiet": True,
            "no_warnings": True,
            "geo_bypass": True,
            "nocheckcertificate": True,
            "cookiefile": COOKIES_FILE_PATH if os.path.exists(COOKIES_FILE_PATH) else None,
            "outtmpl": os.path.join(DOWNLOAD_DIR, "%(id)s.%(ext)s"),
        }

        if selected_mode == "video":
            opts["format"] = "bestvideo[height<=?720][ext=mp4]+bestaudio[ext=m4a]/best[ext=mp4]"
            opts["merge_output_format"] = "mp4"
        elif selected_mode == "m4a":
            opts["format"] = "bestaudio[ext=m4a]/best"
        elif selected_mode == "song_video":
            opts["format"] = "bestvideo+bestaudio/best"
            opts["merge_output_format"] = "mp4"
        elif selected_mode == "song_audio":
            opts["format"] = "bestaudio/best"
            opts["postprocessors"] = [{
                "key": "FFmpegExtractAudio",
                "preferredcodec": "mp3",
                "preferredquality": "128",
            }]
        else:
            opts["format"] = "bestaudio/best"
            opts["postprocessors"] = [{
                "key": "FFmpegExtractAudio",
                "preferredcodec": "m4a",
                "preferredquality": "128",
            }]

        with yt_dlp.YoutubeDL(opts) as ydl:
            info = ydl.extract_info(url, download=False)
            video_id = info['id']
            
            existing_files = glob.glob(os.path.join(DOWNLOAD_DIR, f"{video_id}.*"))
            if existing_files:
                return existing_files[0]

            downloaded_info = ydl.extract_info(url, download=True)
            final_path = ydl.prepare_filename(downloaded_info)
            
            if "postprocessors" in opts:
                ext = opts["postprocessors"][0]["preferredcodec"]
                final_path = os.path.splitext(final_path)[0] + f".{ext}"
            
            return final_path

    try:
        result_path = await loop.run_in_executor(None, download_sync)
        if result_path and os.path.exists(result_path):
            return 1, result_path
        return 0, "File download failed"
    except Exception as e:
        logger.error(f"Download Error: {e}")
        return 0, str(e)

async def cleanup_file(filepath: str, delay: int = 1800):
    await asyncio.sleep(delay)
    try:
        if os.path.exists(filepath):
            os.remove(filepath)
    except:
        pass

async def get_playlist_ids(link, limit):
    command_args = ["yt-dlp", "-i", "--get-id", "--flat-playlist", "--playlist-end", str(limit), "--js-runtime", "node", "--skip-download", link]
    if os.path.exists(COOKIES_FILE_PATH):
        command_args.extend(["--cookies", COOKIES_FILE_PATH])
    
    process = await asyncio.create_subprocess_exec(
        *command_args,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE
    )
    stdout, _ = await process.communicate()
    try:
        return [key for key in stdout.decode().split("\n") if key.strip() != ""]
    except:
        return []
            



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import json
from pathlib import Path
import logging
import os
import re
import sys
import time
from asyncio import get_event_loop
from base64 import b64decode
from distutils.util import strtobool as sb
from logging import DEBUG, INFO, basicConfig, getLogger
from math import ceil
from pathlib import Path
from sys import version_info

from dotenv import load_dotenv
from git import Repo
from pytgcalls import PyTgCalls
from requests import get
from telethon import Button
from telethon.errors import UserIsBlockedError
from telethon.network.connection.tcpabridged import ConnectionTcpAbridged
from telethon.sessions import StringSession
from telethon.sync import TelegramClient, custom, events
from telethon.tl.types import InputWebDocument
from telethon.utils import get_display_name


def STORAGE(n):
    return Storage(Path("data") / n)
    
FILE_NAME = "data.json"


class Storage:
    class _Guard:
        def __init__(self, storage):
            self._storage = storage

        def __enter__(self):
            self._storage._autosave = False

        def __exit__(self, *args):
            self._storage._autosave = True
            self._storage._save()

    def __init__(self, root):
        self._root = Path(root)
        self._autosave = True
        self._guard = self._Guard(self)
        if (self._root / FILE_NAME).is_file():
            with open(self._root / FILE_NAME) as file_pointer:
                self._data = json.load(file_pointer)
        else:
            self._data = {}

    def bulk_save(self):
        return self._guard

    def __getattr__(self, name):
        if name.startswith("_"):
            raise ValueError("Anda hanya dapat mengakses anggota pribadi yang ada")
        return self._data.get(name, None)

    def __setattr__(self, name, value):
        if name.startswith("_"):
            self.__dict__[name] = value
        else:
            self._data[name] = value
            if self._autosave:
                self._save()

    def _save(self):
        if not self._root.is_dir():
            self._root(parents=True, exist_ok=True)
        with open(self._root / FILE_NAME, "w") as file_pointer:
            json.dump(self._data, file_pointer)



import sys
from decouple import config

try:
    from dotenv import load_dotenv
    load_dotenv()
except ImportError:
    pass


class Var:
    # --- MANDATORY (UTAMA) ---
    API_ID = (
        int(sys.argv[1]) if len(sys.argv) > 1 else config("API_ID", default=6, cast=int)
    )
    API_HASH = (
        sys.argv[2]
        if len(sys.argv) > 2
        else config("API_HASH", default="eb06d4abfb49dc3eeb1aeb98ae0f581e")
    )
    SESSION = sys.argv[3] if len(sys.argv) > 3 else config("SESSION", default=None)
        
    # --- MULTI CLIENT ---
    API_ID2 = config("API_ID2", default=None) 
    if API_ID2 is not None:
        API_ID2 = int(API_ID2)
    API_HASH2 = config("API_HASH2", default=None) 
    SESSION2 = config("SESSION2", default=None)
    
    API_ID3 = config("API_ID3", default=None) 
    if API_ID3 is not None:
        API_ID3 = int(API_ID3)
    API_HASH3 = config("API_HASH3", default=None)
    SESSION3 = config("SESSION3", default=None)
    
    # --- DATABASE (REDIS) ---
    REDIS_URI = (
        sys.argv[4]
        if len(sys.argv) > 4
        else (config("REDIS_URI", default=None) or config("REDIS_URL", default=None))
    )
    REDIS_PASSWORD = (
        sys.argv[5] if len(sys.argv) > 5 else config("REDIS_PASSWORD", default=None)
    )
    # Railway/Other Redis Compatibility
    REDISPASSWORD = config("REDISPASSWORD", default=None)
    REDISHOST = config("REDISHOST", default=None)
    REDISPORT = config("REDISPORT", default=None)
    REDISUSER = config("REDISUSER", default=None)
    
    # --- DATABASE (SQL & MONGO) ---
    DATABASE_URL = config("DATABASE_URL", default=None)
    MONGO_URI = config("MONGO_URI", default=None)

    # VC BOT CONFIGURATION
    VCBOT = config("VCBOT", default=False, cast=bool)
    VC_SESSION = config("VC_SESSION", default=None)
    VC_SESSION1 = config("VC_SESSION1", default=None)
    VC_SESSION2 = config("VC_SESSION2", default=None)
    VC_SESSION3 = config("VC_SESSION3", default=None)
    VC_SESSION4 = config("VC_SESSION4", default=None)
    VC_SESSION5 = config("VC_SESSION5", default=None)

    # --- EXTRAS & BOT SETTINGS ---
    BOT_TOKEN = config("BOT_TOKEN", default=None)
    LOG_CHANNEL = config("LOG_CHANNEL", default=0, cast=int)
    HEROKU_APP_NAME = config("HEROKU_APP_NAME", default=None)
    HEROKU_API = config("HEROKU_API", default=None)
    ADDONS = config("ADDONS", default=False, cast=bool)
    
    ASSISTANT_ID = (
        int(sys.argv[1]) if len(sys.argv) > 1 else config("ASSISTANT_ID", default=1012838012, cast=int)
    )
    CMD_IMG = config("CMD_IMG", default="https://telegra.ph/file/66518ed54301654f0b126.png")
    



from .. import run_as_module
import resources
from secrets import choice
import random
from secrets import choice
from telethon.tl.types import InputMessagesFilterVideo, InputMessagesFilterVoice
from telethon.tl.types import InputMessagesFilterPhotos
#from . import eor, ultroid_cmd, get_string, OWNER_NAME
from telethon import Button
if not run_as_module:
    from ..exceptions import RunningAsFunctionLibError

    raise RunningAsFunctionLibError(
        "You are running 'xteam' as a functions lib, not as run module. You can't access this folder.."
    )

from .. import *
from . import *

DEVLIST = [
    1187804106, # @xteam_clone
    719195224,  # @xditya
    1322549723,  # @danish_00
    1903729401,  # @its_buddhhu
    1303895686,  # @Sipak_OP
    611816596,  # @Arnab431
    1318486004,  # @sppidy
    803243487,  # @hellboi_atul
]

devs = [
    1434595544, # @xteam_clone
    719195224,  # @xditya
    1322549723,  # @danish_00
    1903729401,  # @its_buddhhu
    1303895686,  # @Sipak_OP
    611816596,  # @Arnab431
    1318486004,  # @sppidy
    803243487,  # @hellboi_atul
]


ULTROID_IMAGES = [
    f"https://graph.org/file/{_}.jpg"
    for _ in [
        "8d7b534e34e13316a7dd2"
        #"ec250c66268b62ee4ade6",
        #"3c25230ae30d246194eba",
        #"b01715a61b9e876c0d45d",
        #"4ceaf720a96a24527ecff",
        #"a96223b574f29f3f0d184",
        #"6e081d339a01cc6190393",
    ]
]

XTEAM_URBOT = [
    f"XTEAM-URBOT is a versatile and feature-rich Telegram userbot built using the Telethon library in Python.\n\n" \
    f"Think of it as a customizable bot that runs on your Telegram account, allowing you to automate tasks, manage your groups more effectively, and add a variety of fun and useful features to your Telegram experience.\n" \
    f"• xteam-urbot interesting:\n" \
    f"• Extensive Functionality: xteam-urbot boasts a wide array of built-in modules (plugins) that can perform various actions." f"These can include things like:\n" \
    f"* Moderation tools: Anti-flood, ban/kick/mute users, manage messages.\n" \
    f"* Utility functions: Searching the web, calculating, converting currencies, getting weather information.\n" \
    f"* Media manipulation: Downloading media, converting file formats.\n" \
    f"* Fun commands: Sending memes, playing games.\n" \
    f"* Automation: Setting up timed messages, reacting to specific triggers.\n" \
    f"* Customization: One of the key strengths of xteam-urbot is its modular design.\n" \
    f"You can choose which modules to enable or disable based on your needs." f"Furthermore, it supports adding third-party plugins and even developing your own.\n" \
    f"* Community-Driven: xteam-urbot is actively developed by the Teamxteam-urbot on GitHub, which means it's constantly being updated with new features and improvements." f"There's also a community of users who share plugins and provide support.\n" \
    f"* Self-Hosted: Unlike regular Telegram bots that run on separate servers, a userbot like xteam-urbot runs directly on your device (or a server you control) using your Telegram account." f"This gives you more control but also requires you to handle the setup and running of the bot.\n" \
    f"• If you're considering using xteam-urbot, here are a few things to keep in mind:\n" \
    f"• Technical Skills: Setting up and managing a userbot like xteam-urbot typically requires some familiarity with the command line and Python.\n" \
    f"* Telegram's Terms of Service: While userbots can be powerful, it's crucial to use them responsibly and be aware of Telegram's terms of service to avoid any potential issues with your account.\n" \
    f"Avoid using them for spamming or other abusive activities.\n" \
    f"* Resource Usage: Running a userbot can consume resources on your device or server.\n" \
    f"• Where to find more information:\n" \
    f"• GitHub: The main repository for xteam-urbot is on GitHub at https://github.com/xteam-cloner/xteam-urbot." f"You'll find the source code, installation instructions, and documentation there.\n" \
    f"• TeamX Organization: You can explore other related projects and plugins under the xteam-urbot organization on GitHub: https://github.com/xteam-cloner.\n" \
    f"• In short, xteam-urbot is a powerful tool for Telegram users who want to extend the platform's functionality and automate tasks." f"However, it comes with a learning curve and requires responsible usage.\n" \
]

ALIVE_TEXT = [
    f"XTEAM-URBOT is a versatile and feature-rich Telegram userbot built using the Telethon library in Python.\n\n" \
    f"Think of it as a customizable bot that runs on your Telegram account, allowing you to automate tasks, manage your groups more effectively, and add a variety of fun and useful features to your Telegram experience.\n" \
    f"• xteam-urbot interesting:\n" \
    f"• Extensive Functionality: xteam-urbot boasts a wide array of built-in modules (plugins) that can perform various actions." f"These can include things like:\n" \
    f"* Moderation tools: Anti-flood, ban/kick/mute users, manage messages.\n" \
    f"* Utility functions: Searching the web, calculating, converting currencies, getting weather information.\n" \
    f"* Media manipulation: Downloading media, converting file formats.\n" \
    f"* Fun commands: Sending memes, playing games.\n" \
    f"* Automation: Setting up timed messages, reacting to specific triggers.\n" \
    f"* Customization: One of the key strengths of xteam-urbot is its modular design.\n" \
    f"You can choose which modules to enable or disable based on your needs." f"Furthermore, it supports adding third-party plugins and even developing your own.\n" \
    f"* Community-Driven: xteam-urbot is actively developed by the Teamxteam-urbot on GitHub, which means it's constantly being updated with new features and improvements." f"There's also a community of users who share plugins and provide support.\n" \
    f"* Self-Hosted: Unlike regular Telegram bots that run on separate servers, a userbot like xteam-urbot runs directly on your device (or a server you control) using your Telegram account." f"This gives you more control but also requires you to handle the setup and running of the bot.\n" \
    f"• If you're considering using xteam-urbot, here are a few things to keep in mind:\n" \
    f"• Technical Skills: Setting up and managing a userbot like xteam-urbot typically requires some familiarity with the command line and Python.\n" \
    f"* Telegram's Terms of Service: While userbots can be powerful, it's crucial to use them responsibly and be aware of Telegram's terms of service to avoid any potential issues with your account.\n" \
    f"Avoid using them for spamming or other abusive activities.\n" \
    f"* Resource Usage: Running a userbot can consume resources on your device or server.\n" \
    f"• Where to find more information:\n" \
    f"• GitHub: The main repository for xteam-urbot is on GitHub at https://github.com/xteam-cloner/xteam-urbot." f"You'll find the source code, installation instructions, and documentation there.\n" \
    f"• TeamX Organization: You can explore other related projects and plugins under the xteam-urbot organization on GitHub: https://github.com/xteam-cloner.\n" \
    f"• In short, xteam-urbot is a powerful tool for Telegram users who want to extend the platform's functionality and automate tasks." f"However, it comes with a learning curve and requires responsible usage.\n" \
]


ALIVE_NAME = [
"❍❍❍❍❍❍❍❍❍❍❍❍❍❍❍❍"
]

PING_IMAGES = [f"/resources/extras/ping_pic.mp4"]

stickers = [
    "CAADAQADeAIAAm_BZBQh8owdViocCAI",
    "CAADAQADegIAAm_BZBQ6j8GpKtnrSgI",
    "CAADAQADfAIAAm_BZBQpqC84n9JNXgI",
    "CAADAQADfgIAAm_BZBSxLmTyuHvlzgI",
    "CAADAQADgAIAAm_BZBQ3TZaueMkS-gI",
    "CAADAQADggIAAm_BZBTPcbJMorVVsQI",
    "CAADAQADhAIAAm_BZBR3lnMZRdsYxAI",
    "CAADAQADhgIAAm_BZBQGQRx4iaM4pQI",
    "CAADAQADiAIAAm_BZBRRF-cjJi_QywI",
    "CAADAQADigIAAm_BZBQQJwfzkqLM0wI",
    "CAADAQADjAIAAm_BZBQSl5GSAT0viwI",
    "CAADAQADjgIAAm_BZBQ2xU688gfHhQI",
    "CAADAQADkAIAAm_BZBRGuPNgVvkoHQI",
    "CAADAQADpgIAAm_BZBQAAZr0SJ5EKtQC",
    "CAADAQADkgIAAm_BZBTvuxuayqvjhgI",
    "CAADAQADlAIAAm_BZBSMZdWN2Yew1AI",
    "CAADAQADlQIAAm_BZBRXyadiwWGNkwI",
    "CAADAQADmAIAAm_BZBQDoB15A1jS1AI",
    "CAADAQADmgIAAm_BZBTnOLQ8_d72vgI",
    "CAADAQADmwIAAm_BZBTve1kgdG0Y5gI",
    "CAADAQADnAIAAm_BZBQUMyFiylJSqQI",
    "CAADAQADnQIAAm_BZBSMAe2V4pwhNgI",
    "CAADAQADngIAAm_BZBQ06D92QL_vywI",
    "CAADAQADnwIAAm_BZBRw7UAbr6vtEgI",
    "CAADAQADoAIAAm_BZBRkv9DnGPXh_wI",
    "CAADAQADoQIAAm_BZBQwI2NgQdyKlwI",
    "CAADAQADogIAAm_BZBRPHJF3XChVLgI",
    "CAADAQADowIAAm_BZBThpas7rZD6DAI",
    "CAADAQADpAIAAm_BZBQcC2DpZcCw1wI",
    "CAADAQADpQIAAm_BZBQKruTcEU4ntwI",
]


#@call_back("asupan")
async def asupannya(event):
    try:
        asupannya = [
            asupan
            async for asupan in event.client.iter_messages(
                "@xcryasupan", filter=InputMessagesFilterVideo
            )
        ]
        await event.client.send_file(
            event.chat_id, file=choice(asupannya), reply_to=event.reply_to_msg_id
        )
        await xx.delete()
    except Exception:
        await xx.edit("**Tidak bisa menemukan video asupan.**")

QUOTES = [
"Nothing is impossible",
"Anyone can be anything",
"Take the risk or lose the chance",
"When there is a will, there is way",
"Do the best and let God do the rest",
"Good things take time",
"Don’t stop when you’re tired,stop when you’re done",
"Our life is very difficult,but there are millions of more difficult life out there",
"In life, there will be things that come up by itself, \nbut there will be things that also need to struggle to get it",
"Love comfort like sunshine after rain",
]



from .. import udB


class KeyManager:
    def __init__(self, key, cast=None) -> None:
        self._key = key
        self._cast = cast

    def get(self):
        _data = udB.get_key(self._key)
        if self._cast and not isinstance(_data, self._cast):
            return [_data] if self._cast == list else self._cast(_data)
        return _data or (self._cast() if callable(self._cast) else self._cast)

    def get_child(self, key):
        return self.get()[key]

    def count(self):
        return len(self.get())

    def add(self, item):
        content = self.get()
        if content == None and callable(type(item)):
            content = type(item)()
        if isinstance(content, dict) and isinstance(item, dict):
            content.update(item)
        elif isinstance(content, list) and item not in content:
            content.append(item)
        else:
            return
        udB.set_key(self._key, content)

    def remove(self, item):
        content = self.get()
        if isinstance(content, list) and item in content:
            content.remove(item)
        elif isinstance(content, dict) and content.get(item):
            del content[item]
        else:
            return
        udB.set_key(self._key, content)

    def contains(self, item):
        return item in self.get()



import os
import logging
from telethon.errors.rpcerrorlist import UserAlreadyParticipantError 
from telethon.tl import functions
from telethon.tl.types import InputPeerChannel, InputPeerChat
from pytgcalls.exceptions import (
    NoActiveGroupCall,
    LiveStreamFound,
)
from ntgcalls import TelegramServerError, ConnectionNotFound 

from pytgcalls.types import MediaStream
from pytgcalls.types.stream import VideoQuality, AudioQuality
from xteam import call_py, LOGS, bot

FILE_PATH = os.path.join(os.getcwd(), 'resources', 'audio-man.mp3') 

async def join_call(chat_id: int, link: str, video: bool = False):
    
    try:
        if video:
            stream = MediaStream(
                media_path=link, 
                audio_parameters=AudioQuality.HIGH,
                video_parameters=VideoQuality.HD_720p,
            )
        else:
            stream = MediaStream(
                media_path=link, 
                audio_parameters=AudioQuality.HIGH,
                video_flags=MediaStream.Flags.IGNORE,
            )
    except Exception as e:
        LOGS.error(f"Gagal menyiapkan MediaStream: {e}")
        return 0

    try:
        await call_py.play(chat_id, stream)
        LOGS.info(f"Playing now!")
        return 1
        
    except NoActiveGroupCall:
        LOGS.warning(f"Tidak ada VC aktif di {chat_id}. Gagal memutar.")
        return 0
        
    except LiveStreamFound:
        LOGS.warning(f"Ditemukan live stream di {link}. Playback gagal.")
        return 0
    
    except ConnectionNotFound:
        LOGS.error(f"Koneksi terputus saat mencoba join VC {chat_id}.")
        return 0

    except TelegramServerError:
        LOGS.error(f"Server Telegram sedang bermasalah di VC {chat_id}.")
        return 0
        
    except Exception as e:
        error_msg = str(e).lower()
        
        if 'already' in error_msg:
             LOGS.warning(f"Asisten sudah ada di VC {chat_id}. Mencoba mengganti stream...")
             try:
                 await call_py.change_stream(chat_id, stream)
                 LOGS.info(f"Stream berhasil diperbarui ke FHD di {chat_id}.")
                 return 1
             except Exception as sub_e:
                 LOGS.error(f"Gagal mengganti stream di VC {chat_id}: {sub_e}")
                 return 0

        LOGS.error(f"Error tidak dikenal di {chat_id}: {e}")
        return 0
        



# Ultroid - UserBot
# Copyright (C) 2021-2023 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/xteam/blob/main/LICENSE>.

import base64
import ipaddress
import struct
import sys

from telethon.errors.rpcerrorlist import AuthKeyDuplicatedError
from telethon.sessions.string import _STRUCT_PREFORMAT, CURRENT_VERSION, StringSession

from xteam.configs import Var
from . import *
from .BaseClient import UltroidClient

_PYRO_FORM = {351: ">B?256sI?", 356: ">B?256sQ?", 362: ">BI?256sQ?"}

# https://github.com/pyrogram/pyrogram/blob/master/docs/source/faq/what-are-the-ip-addresses-of-telegram-data-centers.rst

DC_IPV4 = {
    1: "149.154.175.53",
    2: "149.154.167.51",
    3: "149.154.175.100",
    4: "149.154.167.91",
    5: "91.108.56.130",
}


def validate_session(session, logger=LOGS, _exit=True):
    from strings import get_string

    if session:
        # Telethon Session
        if session.startswith(CURRENT_VERSION):
            if len(session.strip()) != 353:
                logger.exception(get_string("py_c1"))
                sys.exit()
            return StringSession(session)

        # Pyrogram Session
        elif len(session) in _PYRO_FORM.keys():
            data_ = struct.unpack(
                _PYRO_FORM[len(session)],
                base64.urlsafe_b64decode(session + "=" * (-len(session) % 4)),
            )
            if len(session) in [351, 356]:
                auth_id = 2
            else:
                auth_id = 3
            dc_id, auth_key = data_[0], data_[auth_id]
            return StringSession(
                CURRENT_VERSION
                + base64.urlsafe_b64encode(
                    struct.pack(
                        _STRUCT_PREFORMAT.format(4),
                        dc_id,
                        ipaddress.ip_address(DC_IPV4[dc_id]).packed,
                        443,
                        auth_key,
                    )
                ).decode("ascii")
            )
        else:
            logger.exception(get_string("py_c1"))
            if _exit:
                sys.exit()
    logger.exception(get_string("py_c2"))
    if _exit:
        sys.exit()


"""def vc_connection(udB, ultroid_bot):
    from strings import get_string

    VC_SESSION = Var.VC_SESSION or udB.get_key("VC_SESSION")
    if VC_SESSION and VC_SESSION != Var.SESSION:
        LOGS.info("Starting up VcClient.")
        try:
            return UltroidClient(
                validate_session(VC_SESSION, _exit=False),
                log_attempt=False,
                exit_on_error=False,
            )
        except (AuthKeyDuplicatedError, EOFError):
            LOGS.info(get_string("py_c3"))
            udB.del_key("VC_SESSION")
        except Exception as er:
            LOGS.info("While creating Client for VC.")
            LOGS.exception(er)
    return ultroid_bot
"""

"""async def vc_connection(udB, ultroid_bot):
    from strings import get_string
    from telethon import TelegramClient
    from pytgcalls import PyTgCalls

    VC_SESSION = Var.VC_SESSION or udB.get_key("VC_SESSION")
    
    # Klien MTProto untuk Obrolan Suara (menggunakan Telethon)
    vc_client = None 

    if VC_SESSION and VC_SESSION != Var.SESSION:
        LOGS.info("Starting up Telethon Client for VcClient.")
        
        try:
            # 1. Buat Klien MTProto (menggunakan sesi VC)
            # Anda mungkin perlu menyesuaikan cara validasi/pembuatan sesi
            # sesuai dengan Ultroid, tapi intinya adalah mendapatkan TelegramClient.
            vc_client = TelegramClient(
                session=validate_session(VC_SESSION, _exit=False),
                api_id=Var.API_ID,  # Ganti dengan cara Ultroid mendapatkan API_ID/HASH
                api_hash=Var.API_HASH, 
                #log_attempts=False,
                system_version="UltroidVC"
            )
            # Pastikan klien terhubung
            await vc_client.start()
            
            # 2. Buat instance PyTgCalls dengan klien MTProto
            vc_call = PyTgCalls(vc_client)
            await vc_call.start()
            
            LOGS.info("PyTgCalls Client started successfully.")
            return vc_call # Mengembalikan klien PyTgCalls
            
        except (AuthKeyDuplicatedError, EOFError):
            LOGS.info(get_string("py_c3"))
            udB.del_key("VC_SESSION")
        except Exception as er:
            LOGS.info("While creating PyTgCalls Client for VC.")
            LOGS.exception(er)
            
    # Jika gagal atau tidak ada sesi VC, kembalikan bot utama atau None
    #return ultroid_bot # Atau kembalikan None, tergantung alur Ultroid

# Catatan: Fungsi ini perlu menjadi 'async def' karena menggunakan 'await'
# dan PyTgCalls/Telethon bersifat asinkron.
                
"""



import asyncio
from telethon import functions, errors
from telethon.tl.functions.messages import ImportChatInviteRequest, ExportChatInviteRequest
from telethon.tl.functions.channels import InviteToChannelRequest, GetParticipantRequest
from telethon.errors.rpcerrorlist import UserAlreadyParticipantError
from xteam import bot
from xteam.configs import Var

ASSISTANT_ID = Var.ASSISTANT_ID

async def auto_delete(msg, delay=2):
    await asyncio.sleep(delay)
    try:
        await msg.delete()
    except Exception:
        pass

def AssistantAdd(mystic):
    async def wrapper(event):
        if not event.is_group and not event.is_channel:
            return await mystic(event)

        try:
            await event.client(GetParticipantRequest(event.chat_id, ASSISTANT_ID))
            return await mystic(event)
        except (errors.UserNotParticipantError, ValueError):
            try:
                await event.client(InviteToChannelRequest(event.chat_id, [ASSISTANT_ID]))
                status = await event.reply("**ᴀssɪsᴛᴀɴᴛ sᴜᴄᴄᴇssғᴜʟʟʏ ᴀᴅᴅᴇᴅ!**")
                asyncio.create_task(auto_delete(status, 5))
            except errors.ChatAdminRequiredError:
                try:
                    invite = await event.client(ExportChatInviteRequest(event.chat_id))
                    hash_code = invite.link.split('+')[-1] if '+' in invite.link else invite.link.split('/')[-1]
                    await bot(ImportChatInviteRequest(hash_code))
                    status = await event.reply("**ᴀssɪsᴛᴀɴᴛ sᴜᴄᴄᴇssғᴜʟʟʏ ᴊᴏɪɴᴇᴅ!**")
                    asyncio.create_task(auto_delete(status, 5))
                except Exception:
                    error = await event.reply("❌ **ᴇʀʀᴏʀ**: Saya butuh hak admin (Add Users) untuk mengundang assistant!")
                    asyncio.create_task(auto_delete(error, 10))
                    return
            except Exception as e:
                error = await event.reply(f"❌ **ᴀssɪsᴛᴀɴᴛ ғᴀɪʟᴇᴅ ᴛᴏ ᴊᴏɪɴ**\n\n**Note**: `{e}`")
                asyncio.create_task(auto_delete(error, 10))
                return
        except errors.ChatAdminRequiredError:
            return await mystic(event)
        except Exception:
            return await mystic(event)
                
        return await mystic(event)

    return wrapper
                



# Ultroid - UserBot
# Copyright (C) 2021-2025 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.


import os
import sys
import telethonpatch
from .version import __version__

run_as_module = __package__ in sys.argv or sys.argv[0] == "-m"


class ULTConfig:
    lang = "en"
    thumb = "resources/extras/8189450f-de7f-4582-ba94-f8ec2d928b31.jpeg"


if run_as_module:
    import time
    import asyncio

    from typing import Optional
    from pytgcalls import PyTgCalls
    from telethon import TelegramClient
    
    from .configs import Var
    from .startup import *
    from .startup._database import UltroidDB
    from .startup.BaseClient import UltroidClient
    from .startup.connections import validate_session
    from .startup.funcs import _version_changes, autobot, enable_inline, update_envs
    from .version import ultroid_version

    import logging
    logging.getLogger('apscheduler.scheduler').setLevel(logging.ERROR)
    logging.getLogger('apscheduler.executors.default').setLevel(logging.ERROR)

    if not os.path.exists("./plugins"):
        LOGS.error(
            "'plugins' folder not found!\nMake sure that, you are on correct path."
        )
        exit()

    start_time = time.time()
    _ult_cache = {}
    _ignore_eval = []
    
    call_py: Optional[PyTgCalls] = None
    bot : Optional[TelegramClient] = None

    udB = UltroidDB()
    update_envs()

    LOGS.info(f"Connecting to {udB.name}...")
    if udB.ping():
        LOGS.info(f"Connected to {udB.name} Successfully!")

    BOT_MODE = udB.get_key("BOTMODE")
    
    # Force DUAL_MODE to always be True
    DUAL_MODE = True
    USER_MODE = udB.get_key("USER_MODE")

    if BOT_MODE:
        # Menghapus batasan yang mematikan DUAL_MODE saat BOT_MODE aktif
        ultroid_bot = None

        if not udB.get_key("BOT_TOKEN"):
            LOGS.critical(
                '"BOT_TOKEN" not Found! Please add it, in order to use "BOTMODE"'
            )
            sys.exit()
    else:
        ultroid_bot = UltroidClient(
            validate_session(Var.SESSION, LOGS),
            udB=udB,
            app_version=ultroid_version,
            device_model="xteam-musicubot",
        )
        ultroid_bot.run_in_loop(autobot())

    if USER_MODE:
        asst = ultroid_bot
    else:
        client_id = os.environ.get("CLIENT_ID", "1")
        asst_session = f"asst{client_id}"
        asst = UltroidClient(asst_session, bot_token=udB.get_key("BOT_TOKEN"), udB=udB)

    if BOT_MODE:
        ultroid_bot = asst
        if udB.get_key("OWNER_ID"):
            try:
                ultroid_bot.me = ultroid_bot.run_in_loop(
                    ultroid_bot.get_entity(udB.get_key("OWNER_ID"))
                )
            except Exception as er:
                LOGS.exception(er)
    elif not asst.me.bot_inline_placeholder and asst._bot:
        ultroid_bot.run_in_loop(enable_inline(ultroid_bot, asst.me.username))

    _version_changes(udB)

    HNDLR = udB.get_key("HNDLR") or "."
    DUAL_HNDLR = udB.get_key("DUAL_HNDLR") or "/"
    SUDO_HNDLR = udB.get_key("SUDO_HNDLR") or HNDLR
else:
    print("xteam 2022 © Xteam-Cloner")

    import logging
    from logging import getLogger

    LOGS = getLogger("xteam")
    
    logging.basicConfig(level=logging.DEBUG)
    LOGS.setLevel(logging.DEBUG)

    ultroid_bot = asst = udB = bot = call_py = None



# Man - UserBot
# Copyright (c) 2022 Man-Userbot
# Credits: @mrismanaziz || https://github.com/mrismanaziz
#
# This file is a part of < https://github.com/mrismanaziz/Man-Userbot/ >
# t.me/SharingUserbot & t.me/Lunatic0de



from xteam import LOGS, call_py
from xteam.vcbot.queues import QUEUE, clear_queue, get_queue, pop_an_item


async def skip_item(chat_id: int, x: int):
    if chat_id not in QUEUE:
        return 0
    chat_queue = get_queue(chat_id)
    try:
        songname = chat_queue[x][0]
        chat_queue.pop(x)
        return songname
    except Exception as e:
        LOGS.info(str(e))
        return 0


async def skip_current_song(chat_id: int):
    if chat_id not in QUEUE:
        return 0
    chat_queue = get_queue(chat_id)
    if len(chat_queue) == 1:
        await call_py.leave_group_call(chat_id)
        clear_queue(chat_id)
        return 1
    songname = chat_queue[1][0]
    url = chat_queue[1][1]
    link = chat_queue[1][2]
    type = chat_queue[1][3]
    RESOLUSI = chat_queue[1][4]
    if type == "Audio":
        await call_py.change_stream(
            chat_id,
            AudioPiped(
                url,
                HighQualityAudio(),
            ),
        )
    elif type == "Video":
        if RESOLUSI == 720:
            hm = HighQualityVideo()
        elif RESOLUSI == 480:
            hm = MediumQualityVideo()
        elif RESOLUSI == 360:
            hm = LowQualityVideo()
        await call_py.change_stream(
            chat_id, AudioVideoPiped(url, HighQualityAudio(), hm)
        )
    pop_an_item(chat_id)
    return [songname, link, type]
